(* ::Package:: *)

BeginPackage["ClassicalOrbitTransformations`"]
Unprotect@@Names["ClassicalOrbitTransformations`*"];
ClearAll@@Names["ClassicalOrbitTransformations`*"];

KeplerToCartesian::usage="KeplerToCartesian[P_] where P[[1]]=a, P[[2]]=\[Epsilon], P[[3]]=\[Iota], P[[4]]=\[CapitalOmega], P[[5]]=\[Omega], P[[6]]=\[Lambda]"
CartesianToKepler::usage="CartesianToKepler[Q_,P_] where Q[[1]]=x, Q[[2]]=y, Q[[3]]=z, P[[1]]=\!\(\*SubscriptBox[\(v\), \(x\)]\), P[[2]]=\!\(\*SubscriptBox[\(v\), \(y\)]\), P[[3]]=\!\(\*SubscriptBox[\(v\), \(z\)]\)"
KeplerToCartesianC::usage="KeplerToCartesianC[P_] where P[[1]]=a, P[[2]]=\[Epsilon], P[[3]]=\[Iota], P[[4]]=\[CapitalOmega], P[[5]]=\[Omega], P[[6]]=\[Lambda]"
CartesianToKeplerC::usage="CartesianToKeplerC[Q_,P_] where Q[[1]]=x, Q[[2]]=y, Q[[3]]=z, P[[1]]=\!\(\*SubscriptBox[\(v\), \(x\)]\), P[[2]]=\!\(\*SubscriptBox[\(v\), \(y\)]\), P[[3]]=\!\(\*SubscriptBox[\(v\), \(z\)]\)"
Begin["`Private`"]


KeplerToCartesian[P_,GM_]:=Module[{a=P[[1]],\[Epsilon]=P[[2]],\[Iota]=P[[3]],\[CapitalOmega]=P[[4]],\[Omega]=P[[5]],\[Lambda]=P[[6]],\[Mu]=GM,Ea,Ma,p,q,rvec,vvec},
p={Cos[\[Omega]]Cos[\[CapitalOmega]]-Sin[\[Omega]]Cos[\[Iota]]Sin[\[CapitalOmega]],Cos[\[Omega]]Sin[\[CapitalOmega]]+Sin[\[Omega]]Cos[\[Iota]]Cos[\[CapitalOmega]],Sin[\[Omega]]Sin[\[Iota]]};
q={-Sin[\[Omega]]Cos[\[CapitalOmega]]-Cos[\[Omega]]Cos[\[Iota]]Sin[\[CapitalOmega]],-Sin[\[Omega]]Sin[\[CapitalOmega]]+Cos[\[Omega]]Cos[\[Iota]]Cos[\[CapitalOmega]],Sin[\[Iota]]Cos[\[Omega]]};
Ea=If[\[Pi]<\[Lambda]<2\[Pi],2\[Pi]-ArcCos[(\[Epsilon]+Cos[\[Lambda]])/(1+\[Epsilon] Cos[\[Lambda]])],ArcCos[(\[Epsilon]+Cos[\[Lambda]])/(1+\[Epsilon] Cos[\[Lambda]])]];
Ma=Ea-\[Epsilon] Sin[Ea];
rvec=a(Cos[Ea]-\[Epsilon])p+a Sqrt[1-\[Epsilon]^2]Sin[Ea]q;
vvec=1/(1-\[Epsilon] Cos[Ea]) Sqrt[\[Mu]/a](-\[Epsilon] Sin[Ea]p+Sqrt[1-\[Epsilon]^2]Cos[Ea]q);
{rvec,vvec}]



CartesianToKepler[Q_,P_,GM_]:=Module[{\[Mu]=GM,a,\[Epsilon],\[Iota],\[CapitalOmega],\[Omega],\[Lambda],hvec,evec,h,r,nvec,n},
r=Norm[Q];
hvec=Q\[Cross]P;h=Norm[hvec];
evec=1/\[Mu] (P\[Cross]hvec)-Q/r;\[Epsilon]=Norm[evec];
a=hvec.hvec/(\[Mu](1-\[Epsilon]^2));
\[Iota]=ArcCos[hvec[[3]]/h];
nvec={0,0,1}\[Cross]hvec;n=Norm[nvec];\[CapitalOmega]=If[nvec.{0,1,0}<0,2\[Pi]-ArcCos[{1,0,0}.nvec/n],ArcCos[{1,0,0}.nvec/n]];
\[Omega]=If[evec.{0,0,1}<0,2\[Pi]-ArcCos[nvec.evec/(n \[Epsilon])],ArcCos[nvec.evec/(n \[Epsilon])]];
\[Lambda]=If[Q.P<0,2\[Pi]-ArcCos[evec.Q/(\[Epsilon] r)],ArcCos[evec.Q/(\[Epsilon] r)]];
{a,\[Epsilon],180/\[Pi] \[Iota],180/\[Pi] \[CapitalOmega],180/\[Pi] \[Omega],180/\[Pi] \[Lambda]}];


KeplerToCartesianC:=Compile[{{P,_Real,1},{GM,_Real}},
Module[{a=P[[1]],\[Epsilon]=P[[2]],\[Iota]=P[[3]],\[CapitalOmega]=P[[4]],\[Omega]=P[[5]],\[Lambda]=P[[6]],\[Mu]=GM,Ea,Ma,p,q,rvec,vvec},
p={Cos[\[Omega]]Cos[\[CapitalOmega]]-Sin[\[Omega]]Cos[\[Iota]]Sin[\[CapitalOmega]],Cos[\[Omega]]Sin[\[CapitalOmega]]+Sin[\[Omega]]Cos[\[Iota]]Cos[\[CapitalOmega]],Sin[\[Omega]]Sin[\[Iota]]};
q={-Sin[\[Omega]]Cos[\[CapitalOmega]]-Cos[\[Omega]]Cos[\[Iota]]Sin[\[CapitalOmega]],-Sin[\[Omega]]Sin[\[CapitalOmega]]+Cos[\[Omega]]Cos[\[Iota]]Cos[\[CapitalOmega]],Sin[\[Iota]]Cos[\[Omega]]};
Ea=If[\[Pi]<\[Lambda]<2\[Pi],2\[Pi]-ArcCos[(\[Epsilon]+Cos[\[Lambda]])/(1+\[Epsilon] Cos[\[Lambda]])],ArcCos[(\[Epsilon]+Cos[\[Lambda]])/(1+\[Epsilon] Cos[\[Lambda]])]];
Ma=Ea-\[Epsilon] Sin[Ea];
rvec=a(Cos[Ea]-\[Epsilon])p+a Sqrt[1-\[Epsilon]^2]Sin[Ea]q;
vvec=1/(1-\[Epsilon] Cos[Ea]) Sqrt[\[Mu]/a](-\[Epsilon] Sin[Ea]p+Sqrt[1-\[Epsilon]^2]Cos[Ea]q);
{rvec,vvec}],CompilationTarget->"C"]


CartesianToKeplerC:=Compile[{{Q,_Real,1},{P,_Real,1},{GM,_Real}},
Module[{\[Mu]=GM,a,\[Epsilon],\[Iota],\[CapitalOmega],\[Omega],\[Lambda],hvec,evec,h,r,nvec,n},
r=Norm[Q];
hvec=Q\[Cross]P;h=Norm[hvec];
evec=1/\[Mu] (P\[Cross]hvec)-Q/r;\[Epsilon]=Norm[evec];
a=hvec.hvec/(\[Mu](1-\[Epsilon]^2));
\[Iota]=ArcCos[hvec[[3]]/h];
nvec={0,0,1}\[Cross]hvec;n=Norm[nvec];\[CapitalOmega]=If[nvec.{0,1,0}<0,2\[Pi]-ArcCos[{1,0,0}.nvec/n],ArcCos[{1,0,0}.nvec/n]];
\[Omega]=If[evec.{0,0,1}<0,2\[Pi]-ArcCos[nvec.evec/(n \[Epsilon])],ArcCos[nvec.evec/(n \[Epsilon])]];
\[Lambda]=If[Q.P<0,2\[Pi]-ArcCos[evec.Q/(\[Epsilon] r)],ArcCos[evec.Q/(\[Epsilon] r)]];
{a,\[Epsilon],180/\[Pi] \[Iota],180/\[Pi] \[CapitalOmega],180/\[Pi] \[Omega],180/\[Pi] \[Lambda]}],CompilationTarget->"C"];


End[]
Protect@@Names["ClassicalOrbitTransformations`*"];
EndPackage[]

